/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.io.File;
import java.awt.Color;
import java.awt.event.*;
import java.util.List;
import java.util.ArrayList;

/**

*/
@SuppressWarnings("serial")
public class SimpleOrientationDemo extends InteractiveFrame
{
   private boolean face = false;
   private static boolean doBackFaceCulling = true;
   private static boolean frontFacingIsCCW = true;

   private boolean showCamera = false;
   private boolean showMatrix = false;
   private double xRotation = 0.0;
   private double yRotation = 0.0;
   private double zRotation = 0.0;

   private Scene scene;

   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
   */
   public SimpleOrientationDemo(String title, int fbWidth, int fbHeight)
   {
      super(title, fbWidth, fbHeight);

      // Create the Scene object that we shall render
      scene = new Scene();

      // Set up the camera's location.
      scene.camera.viewTranslate(0, 0, 2);

      Model model = new Model();

      // The layout of the following two line of code is meant to
      // suggest the relative location of the five vertices.
      Vertex

             v0=new Vertex(-1, 1, 0),          v1=new Vertex(1, 1, 0),

   v2=new Vertex(-2, 0, 0),   v3=new Vertex(0, 0, 0),   v4=new Vertex(2, 0, 0);

      // Use the layout of the above code to convince yourself of the
      // orientation of the following three triangles with respect to
      // the camera's initial  point-of-view.

      model.addVertex(v0, v1, v2, v3, v4);
      Triangle t1 = new Triangle(model, 2, 3, 0); // CCW (wrt the initial pov)
      Triangle t2 = new Triangle(model, 0, 1, 3); // CW  (wrt the initial pov)
      Triangle t3 = new Triangle(model, 3, 4, 1); // CCW (wrt the initial pov)
      model.addPrimitive(t1, t2, t3);

      // Give the models random colors.
      for (Primitive p : model.primitiveList)
      {
         p.setColorRandom();
      }

      // Add a model to the Scene.
      scene.addPosition(new Position(model));

      // Render again.
      FrameBuffer fb = this.fbp.getFrameBuffer();
      fb.clearFB(Color.black);
      Pipeline.render(scene, fb);
      repaint();
   }



   // Implement part of the KeyListener interface.
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('d' == c)
      {
         Pipeline.debug = ! Pipeline.debug;
       //Pipeline.debug_visible = ! Pipeline.debug_visible;
       //Rasterize.debug = ! Rasterize.debug;
      }
      else if ('g' == c)
      {
         doBackFaceCulling = ! doBackFaceCulling;
         for (Position p : scene.positionList)
            p.model.doBackFaceCulling = doBackFaceCulling;
         System.out.print("Back face culling is ");
         System.out.println(doBackFaceCulling ? "on" : "off");
      }
      else if ('G' == c)
      {
         frontFacingIsCCW = ! frontFacingIsCCW;
         for (Position p : scene.positionList)
            p.model.frontFacingIsCCW = frontFacingIsCCW;
         System.out.print("Front face is ");
         System.out.println(frontFacingIsCCW ? "CCW" : "CW");
      }
      else if ('/' == c)
      {
         face = ! face;
         Model m = scene.positionList.get(0).model;
         List<Primitive> primitiveList = new ArrayList<>();
         if (face)
         {
            for (Primitive p : m.primitiveList)
            {
               Face face = new Face(m, p.indexList);
               primitiveList.add(face);
            }
         }
         else
         {
            for (Primitive p : m.primitiveList)
            {
               Triangle t = new Triangle(m, p.indexList);
               primitiveList.add(t);
            }
         }
         m.primitiveList = primitiveList;
      }
      else if ('c' == c)
      {
         // Change the solid random color of the model.
         scene.positionList.get(0).model.setColorRandom();
      }
      else if ('e' == c)
      {
         // Change the random color of each vertex.
         for (Vertex v : scene.positionList.get(0).model.vertexList)
         {
            v.setColorRandom();
         }
      }
      else if ('E' == c)
      {
         // Change the random color of each primitive.
         for (Primitive p : scene.positionList.get(0).model.primitiveList)
         {
            p.setColorRandom();
         }
      }
      else if ('a' == c)
      {
         Pipeline.doAntialiasing = ! Pipeline.doAntialiasing;
      }
      else if ('p' == c)
      {
         scene.camera.perspective = ! scene.camera.perspective;
      }
      else if ('M' == c)
      {
         showCamera = ! showCamera;
      }
      else if ('m' == c)
      {
         showMatrix = ! showMatrix;
      }
      else if ('x' == c)
      {
         xRotation -= 2.0;
      }
      else if ('X' == c)
      {
         xRotation += 2.0;
      }
      else if ('y' == c)
      {
         yRotation -= 2.0;
      }
      else if ('Y' == c)
      {
         yRotation += 2.0;
      }
      else if ('z' == c)
      {
         zRotation -= 2.0;
      }
      else if ('Z' == c)
      {
         zRotation += 2.0;
      }

      // Set the model-to-view transformation matrix.
      // The order of the transformations is very important!
      Position position = scene.positionList.get(0);
      position.matrix2Identity();
      // Move the model relative to its new position.
      position.rotateX(xRotation);
      position.rotateY(yRotation);
      position.rotateZ(zRotation);

      // Set up the camera's view volume.
      if (scene.camera.perspective)
      {
         scene.camera.projPerspective(-1, 1, -1, 1, 1);
      }
      else
      {
         scene.camera.projOrtho(-1, 1, -1, 1);
      }
      if (showCamera && ('n'==c||'N'==c||'o'==c||'O'==c||'r'==c||'R'==c||'p'==c))
      {
         System.out.print( scene.camera );
      }

      if (showMatrix && ('m'==c||'x'==c||'y'==c||'z'==c
                               ||'X'==c||'Y'==c||'Z'==c))
      {
         System.out.println("xRot = " + xRotation
                        + ", yRot = " + yRotation
                        + ", zRot = " + zRotation);
         System.out.print( position.modelMatrix );
      }

      // Render again.
      FrameBuffer fb = this.fbp.getFrameBuffer();
      fb.clearFB(Color.black);
      Pipeline.render(scene, fb);
      fbp.update();
      repaint();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // Define initial dimensions for a FrameBuffer.
      int width  = 512;
      int height = 512;
      // Create an InteractiveFrame containing a FrameBuffer
      // with the given dimensions. NOTE: We need to call the
      // SimpleOrientationDemo constructor in the Java GUI Event
      // Dispatch Thread, otherwise we get a race condition
      // between the constructor (running in the main() thread)
      // and the very first ComponentEvent (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         new Runnable() // an anonymous inner class constructor
         {
            public void run() // implement the Runnable interface
            {
               // call the constructor that builds the gui
               (
                  new SimpleOrientationDemo("Renderer 12", width, height)
               ).setResizable(false);
            }
         }
      );
   }//main()


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, z/Z, keys to rotate the models around the x, y, z axes.");
      System.out.println("Use the s/S keys to scale the size of the models.");
      System.out.println("Use the 'c' key to change the solid model.");
      System.out.println("Use the 'e' key to change the random vertex colors.");
      System.out.println("Use the 'E' key to change the random primitive colors.");
      System.out.println("Use the 'm' key to toggle showing the Model transformation matrix.");
      System.out.println("Use the 'M' key to toggle showing the Camera's two matrices.");
      System.out.println("Use the 'g' key to toggle back face culling.");
      System.out.println("Use the 'G' key to toggle CW vs CCW.");
      System.out.println("Use the '/' key to toggle between Faces and Triangles.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
