/*

*/

package renderer.models;
import  renderer.scene.*;

/**
   Create a wireframe model of a right square pyramid with its
   base in the xz-plane and its apex on the positive y-axis.
<p>
   See <a href="https://en.wikipedia.org/wiki/Pyramid_(geometry)" target="_top">
                https://en.wikipedia.org/wiki/Pyramid_(geometry)</a>

   @see PyramidFrustum
*/
public class Pyramid extends Model
{
   /**
      Create a right square pyramid with its base in the xz-plane,
      a base side length of 2, height 1, and apex on the positive y-axis.
   */
   public Pyramid( )
   {
      this(2, 1, 15, 4);
   }


   /**
      Create a right square pyramid with its base in the xz-plane,
      a base length of {@code s}, height {@code h}, and apex on the
      positive y-axis.

      @param s  side length of the base in the xz-plane
      @param h  height of the apex on the y-axis
      @param n  number of lines of latitude around the body of the pyramid
      @param k  number of triangles in the triangle fan at the top of each side
   */
   public Pyramid(double s, double h, int n, int k)
   {
      super();

      if (n < 1) n = 1;
      if (k < 1) k = 1;

      // Create the pyramid's geometry.

      // Create all the lines of "longitude" from the apex, down
      // to the base, across the base, and then back up to the apex.
      s = s/2;
      double delta = (2 * s) / k;
      // lines of "longitude" perpendicular to the x-axis
      for (int j = 0; j < k + 1; j++)
      {
         double d = j * delta;
         addLineSegment(new Vertex(   0, h,  0), new Vertex(-s+d, 0, -s));
         addLineSegment(new Vertex(-s+d, 0, -s), new Vertex(-s+d, 0,  s));
         addLineSegment(new Vertex(-s+d, 0,  s), new Vertex(   0, h,  0));
      }
      // lines of "longitude" perpendicular to the z-axis
      for (int j = 1; j < k; j++)
      {
         double d = j * delta;
         addLineSegment(new Vertex( 0, h,    0), new Vertex( s, 0, -s+d));
         addLineSegment(new Vertex( s, 0, -s+d), new Vertex(-s, 0, -s+d));
         addLineSegment(new Vertex(-s, 0, -s+d), new Vertex( 0, h,    0));
      }
      // Create all the lines of "latitude" around the pyramid, starting
      // from the base and working upwards.
      double deltaH = h / n;
      double deltaS = s / n;
      for (int i = 0; i < n; i++)
      {
         h = i * deltaH;
         addLineSegment(new Vertex( s, h,  s), new Vertex( s, h, -s));
         addLineSegment(new Vertex( s, h, -s), new Vertex(-s, h, -s));
         addLineSegment(new Vertex(-s, h, -s), new Vertex(-s, h,  s));
         addLineSegment(new Vertex(-s, h,  s), new Vertex( s, h,  s));
         s -= deltaS;
      }
   }
}//Pyramid
