/*

*/

package renderer.models_F;
import  renderer.scene.*;

/**
   Create a wireframe model of a regular dodecahedron
   with its center at the origin, having edge length
   <pre>{@code
     2*(sqrt(5)-1)/(1+sqrt(5)) = (1/2)*(sqrt(5)-1)^2 = 0.7639,
   }</pre>
   and with its vertices on a sphere of radius
   <pre>{@code
     2*sqrt(3)/(1+sqrt(5)) = 1.0705.
   }</pre>
<p>
   See <a href="https://en.wikipedia.org/wiki/Regular_dodecahedron" target="_top">
                https://en.wikipedia.org/wiki/Regular_dodecahedron</a>

   @see Tetrahedron
   @see Cube
   @see Octahedron
   @see Icosahedron
*/
public class Dodecahedron extends Model
{
   /**
      Create a regular dodecahedron with its center at
      the origin, having edge length
      <pre>{@code
        2*(sqrt(5)-1)/(1+sqrt(5)) = (1/2)*(sqrt(5)-1)^2 = 0.7639,
      }</pre>
      and with its vertices on a sphere of radius
      <pre>{@code
        2*sqrt(3)/(1+sqrt(5)) = 1.0705.
     }</pre>
   */
   public Dodecahedron()
   {
      super();

      // Create the dodecahedron's geometry.
      // It has 20 vertices, 30 edgee, and 12 faces.
      double t = (1 + Math.sqrt(5))/2;   // golden ratio
      double r = 1/t;
      double r2 = r * r;
      Vertex[] v = new Vertex[20];
      //https://en.wikipedia.org/wiki/Regular_dodecahedron#Cartesian_coordinates
      // (r, r, r)
      v[0] = new Vertex(-r, -r, -r);
      v[1] = new Vertex(-r, -r,  r);
      v[2] = new Vertex(-r,  r, -r);
      v[3] = new Vertex(-r,  r,  r);
      v[4] = new Vertex( r, -r, -r);
      v[5] = new Vertex( r, -r,  r);
      v[6] = new Vertex( r,  r, -r);
      v[7] = new Vertex( r,  r,  r);

      // (0, r2, 1)
      v[8] = new Vertex( 0, -r2, -1);
      v[9] = new Vertex( 0, -r2,  1);
     v[10] = new Vertex( 0,  r2, -1);
     v[11] = new Vertex( 0,  r2,  1);

      // (r2, 1, 0)
     v[12] = new Vertex(-r2, -1,  0);
     v[13] = new Vertex(-r2,  1,  0);
     v[14] = new Vertex( r2, -1,  0);
     v[15] = new Vertex( r2,  1,  0);

      // (1, 0, r2)
     v[16] = new Vertex(-1,  0, -r2);
     v[17] = new Vertex( 1,  0, -r2);
     v[18] = new Vertex(-1,  0,  r2);
     v[19] = new Vertex( 1,  0,  r2);
/*
      // These vertices create a dodecahedron with vertices
      // on a sphere of radius sqrt(3), and with edge length
      //    2/t = 4/(1 + sqrt(5)) = sqrt(5) - 1 = 1.2361.
      //https://en.wikipedia.org/wiki/Regular_dodecahedron#Cartesian_coordinates
      // (1, 1, 1)
      v[0] = new Vertex(-1, -1, -1);
      v[1] = new Vertex(-1, -1,  1);
      v[2] = new Vertex(-1,  1, -1);
      v[3] = new Vertex(-1,  1,  1);
      v[4] = new Vertex( 1, -1, -1);
      v[5] = new Vertex( 1, -1,  1);
      v[6] = new Vertex( 1,  1, -1);
      v[7] = new Vertex( 1,  1,  1);

      // (0, r, t)
      v[8] = new Vertex( 0, -r, -t);
      v[9] = new Vertex( 0, -r,  t);
     v[10] = new Vertex( 0,  r, -t);
     v[11] = new Vertex( 0,  r,  t);

      // (r, t, 0)
     v[12] = new Vertex(-r, -t,  0);
     v[13] = new Vertex(-r,  t,  0);
     v[14] = new Vertex( r, -t,  0);
     v[15] = new Vertex( r,  t,  0);

      // (t, 0, r)
     v[16] = new Vertex(-t,  0, -r);
     v[17] = new Vertex( t,  0, -r);
     v[18] = new Vertex(-t,  0,  r);
     v[19] = new Vertex( t,  0,  r);
*/
      // Create 12 faces.
//https://github.com/mrdoob/three.js/blob/master/src/geometries/DodecahedronGeometry.js
      addFace(v[ 3], v[11], v[ 7], v[15], v[13]);
      addFace(v[ 7], v[19], v[17], v[ 6], v[15]);
      addFace(v[17], v[ 4], v[ 8], v[10], v[ 6]);
      addFace(v[ 8], v[ 0], v[16], v[ 2], v[10]);
      addFace(v[ 0], v[12], v[ 1], v[18], v[16]);
      addFace(v[ 6], v[10], v[ 2], v[13], v[15]);
      addFace(v[ 2], v[16], v[18], v[ 3], v[13]);
      addFace(v[18], v[ 1], v[ 9], v[11], v[ 3]);
      addFace(v[ 4], v[14], v[12], v[ 0], v[ 8]);
      addFace(v[11], v[ 9], v[ 5], v[19], v[ 7]);
      addFace(v[19], v[ 5], v[14], v[ 4], v[17]);
      addFace(v[ 1], v[12], v[14], v[ 5], v[ 9]);
   }
}//Dodecahedron
