/*

*/

package renderer.models_F;
import  renderer.scene.*;

/**
   Create a solid model of a frustum of a right square pyramid
   with its base in the xz-plane.
<p>
   See <a href="https://en.wikipedia.org/wiki/Frustum" target="_top">
                https://en.wikipedia.org/wiki/Frustum</a>

   @see Pyramid
*/
public class PyramidFrustum extends Model
{
   /**
      Create a frustum of a right square pyramid with its base in the
      xz-plane, a base side length of 2, top side length of 1, and height 1/2.
   */
   public PyramidFrustum( )
   {
      this(2.0, 1.0, 0.5, 7, 4);
   }


   /**
      Create a frustum of a right square pyramid with its base in the
      xz-plane, a base side length of {@code s1}, top side length of
      {@code s2}, and height {@code h}.
   <p>
      Notice that this model works with either {@code s1 > s2} or
      {@code s1 < s2}. In other words, the frustum can have its "apex"
      either above or below the xz-plane.

      @param s1  side length of the base of the frustum
      @param s2  side length of the top of the frustum
      @param h   height of the frustum
      @param n   number of lines of latitude
      @param k   number of lines of longitude
   */
   public PyramidFrustum(double s1, double s2, double h, int n, int k)
   {
      super();

      if (n < 0) n = 0;
      if (k < 1) k = 1;

      // Create the frustum's geometry.

      // An array of vertices to be used to create faces.
      Vertex[] v = new Vertex[8];

      v[0] = new Vertex(-s1/2, 0, -s1/2); // 4 vertices around the base
      v[1] = new Vertex(-s1/2, 0,  s1/2);
      v[2] = new Vertex( s1/2, 0,  s1/2);
      v[3] = new Vertex( s1/2, 0, -s1/2);
      v[4] = new Vertex(-s2/2, h, -s2/2); // 4 vertices around the top
      v[5] = new Vertex(-s2/2, h,  s2/2);
      v[6] = new Vertex( s2/2, h,  s2/2);
      v[7] = new Vertex( s2/2, h, -s2/2);

      // Create 6 faces.
      addFace(v[0], v[3], v[2], v[1]); // base
      addFace(v[4], v[0], v[1], v[5]); // sides
      addFace(v[5], v[1], v[2], v[6]);
      addFace(v[6], v[2], v[3], v[7]);
      addFace(v[7], v[3], v[0], v[4]);
      addFace(v[4], v[5], v[6], v[7]); // top
   }
}//PyramidFrustum
