/*

*/

package renderer.models_F;
import  renderer.scene.*;

/**
   Create a wireframe model of a right equilateral triangular prism
   with the y-axis as its central axis.
<p>
   See <a href="https://en.wikipedia.org/wiki/Triangular_prism" target="_top">
                https://en.wikipedia.org/wiki/Triangular_prism</a>
<p>
   See <a href="https://en.wikipedia.org/wiki/Prism_(geometry)" target="_top">
                https://en.wikipedia.org/wiki/Prism_(geometry)</a>
<p>
   Attach to each triangular end of the prism a tetrahedron.
*/
public class TriangularPrism extends Model
{
   /**
      Create a right equilateral triangular prism with a
      regular tetrahedrons attached to each end so that
      the total length runs from -1 to 1 along the y-axis.
   */
   public TriangularPrism( )
   {
      this(0.5, 0.6);
   }


   /**
      Create an equilateral triangular prism that runs
      from {@code -h} to {@code h} along the y-axis, has
      triangle side length {@code s}, and has a regular
      tetrahedron attached to each end.

      @param s  the length of the triangle's sides
      @param h  the body of the prism runs from -h to h along the y-axis
   */
   public TriangularPrism(double s, double h)
   {
      this(s, h, 0);
   }


   /**
      Create an equilateral triangular prism that runs
      from {@code -h} to {@code h} along the y-axis, has
      triangle side length {@code s}, has a regular
      tetrahedron attached to each end, and has {@code n}
      lines of latitude around the body of the prism.

      @param s  the length of the triangle's sides
      @param h  the body of the prism runs from -h to h along the y-axis
      @param n  number of lines of latitude around the body of the prism
   */
   public TriangularPrism(double s, double h, int n)
   {
      this(s/Math.sqrt(3), h, Math.atan(Math.sqrt(2)), n);
   }


   /**
      Create an equilateral triangular prism that runs
      from {@code -h} to {@code h} along the y-axis, with
      the triangle inscribed in a circle of radius {@code r},
      has a tetrahedron attached to each end where the
      face-edge-face angle of each tetrahedron is {@code theta}
      (with theta in radians!), and has {@code n} lines of
      latitude around the body of the prism.
   <p>
      If {@code theta = 0}, then there are no tetrahedrons at the ends of the prism.
   <p>
      If {@code theta = arctan(sqrt(2)) = 54.736}, then the tetrahedrons are regular.

      @param r      radius of circle in xz-plane that the equilateral triangle is inscribed in
      @param h      the body of the prism runs from -h to h along the y-axis
      @param theta  slant angle of each tetrahedron at the ends of the prism
      @param n      number of lines of latitude around the body of the prism
   */
   public TriangularPrism(double r, double h, double theta, int n)
   {
      this(r, h, r*Math.tan(theta), n, true);
   }


   /**
      Create an equilateral triangular prism that runs
      from {@code -h} to {@code h} along the y-axis, with
      the triangle inscribed in a circle of radius {@code r},
      has a tetrahedron attached to each end where the height
      of each tetrahedron is {@code h2}, and has {@code n} lines
      of latitude around the body of the prism.
   <p>
      So the total height is {@code 2*(h + h2)}.

      @param r   radius of circle in xz-plane that the equilateral triangle is inscribed in
      @param h   the body of the prism runs from h to -h in the y-direction
      @param h2  height of each tetrahedron at the ends of the prism
      @param n   number of lines of latitude around the body of the prism
      @param bothHalves  determines if both halves or only the top half gets created
   */
   public TriangularPrism(double r, double h, double h2, int n, boolean bothHalves)
   {
      super();

      if (n < 0) n = 0;

      // Create the prism's geometry.
      // An array of vertices to be used to create triangles.
      Vertex[][] v = new Vertex[n+2][3];

      double sqrt3 = Math.sqrt(3.0);
      double delta_y = 2.0*h/(n+1);
      if (! bothHalves)  // cut off the bottom half
      {
         delta_y = h/(n+1);
      }

      for (int i = 0; i < n+2; i++)
      {
         double y = -h + i * delta_y;
         if (! bothHalves)  // cut off the bottom half
         {
            y = i * (delta_y / 2);
         }
         v[i][0] = new Vertex( r,    y,    0        );
         v[i][1] = new Vertex(-r/2,  y,  r*0.5*sqrt3);
         v[i][2] = new Vertex(-r/2,  y, -r*0.5*sqrt3);
      }
      Vertex top = new Vertex(0,  h+h2, 0);  // vertex at the top
      Vertex bot = new Vertex(0, -h-h2, 0);  // vertex at the bottom
      if (! bothHalves)  // cut off the bottom half
      {
         bot = new Vertex(0, 0, 0);   // vertex at the bottom
      }

      // 3 top faces
      addFace(top, v[n+1][1], v[n+1][0]);
      addFace(top, v[n+1][2], v[n+1][1]);
      addFace(top, v[n+1][0], v[n+1][2]);
      // 3 bottom faces
      addFace(bot, v[0][0], v[0][1]);
      addFace(bot, v[0][1], v[0][2]);
      addFace(bot, v[0][2], v[0][0]);
      // Create all the square strips along the prism wall.
      for (int i = 0; i < n+1; i++)
      {
         addFace(v[i][0], v[i+1][0], v[i+1][1], v[i][1]);
         addFace(v[i][1], v[i+1][1], v[i+1][2], v[i][2]);
         addFace(v[i][2], v[i+1][2], v[i+1][0], v[i][0]);
      }
   }
}//TriangularPrism
