/*

*/

package renderer.models_T;
import  renderer.scene.*;

/**
   Create a solid model of a cube aligned with
   the x, y, and z axes and with one corner at the
   origin.
<p>
   Here is a picture showing how the cube's eight vertices
   are labeled.
<pre>{@code
                  v[4]
                   +-----------------+ v[5]
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[7] +-----------------+ v[6]  |
           |       |         |       |               y
           |       |         |       |               |
           |       |         |       |               |
           |  v[0] +---------|-------+ v[1]          |
           |      /          |      /                |
           |    /            |    /                  +----. x
           |  /              |  /                   /
           |/                |/                    /
           +-----------------+                    /
          v[3]              v[2]                 z
}</pre>

   @see Cube
*/
public class Box extends Model
{
   /**
      Create a cube with all three sides of length 1.
   */
   public Box( )
   {
      this(1, 1, 1);
   }


   /**
      Create a cube with the given side lengths.

      @param xs  the size of the cube along the x-axis
      @param ys  the size of the cube along the y-axis
      @param zs  the size of the cube along the z-axis
   */
   public Box(double xs, double ys, double zs)
   {
      super();

      // Create the cube's geometry.
      Vertex[] v = new Vertex[8];
      v[0] = new Vertex(0,    0,    0);      // four vertices around the bottom face
      v[1] = new Vertex(0+xs, 0,    0);
      v[2] = new Vertex(0+xs, 0,    0+zs);
      v[3] = new Vertex(0,    0,    0+zs);
      v[4] = new Vertex(0,    0+ys, 0);      // four vertices around the top face
      v[5] = new Vertex(0+xs, 0+ys, 0);
      v[6] = new Vertex(0+xs, 0+ys, 0+zs);
      v[7] = new Vertex(0,    0+ys, 0+zs);

      // Create 12 triangles, 2 for each face of the cube.
      // bottom face
      addTriangle(v[0], v[1], v[2]);
      addTriangle(v[2], v[3], v[0]);
      // front face
      addTriangle(v[3], v[2], v[6]);
      addTriangle(v[6], v[7], v[3]);
      // right face
      addTriangle(v[2], v[1], v[5]);
      addTriangle(v[5], v[6], v[2]);
      // back face
      addTriangle(v[1], v[0], v[4]);
      addTriangle(v[4], v[5], v[1]);
      // left face
      addTriangle(v[0], v[3], v[7]);
      addTriangle(v[7], v[4], v[0]);
      // top face
      addTriangle(v[7], v[6], v[5]);
      addTriangle(v[5], v[4], v[7]);
   }
}//Box
