/*

*/

package renderer.models_T;
import  renderer.scene.*;

/**
   Create a solid model of a regular icosahedron
   with its center at the origin, having edge length
   <pre>{@code
     4/(1+sqrt(5)) = 1.2361,
   }</pre>
   and with its vertices on a sphere of radius
   <pre>{@code
     4/(1+sqrt(5)) * sin(2Pi/5) = 1.1756.
   }</pre>
<p>
   See <a href="https://en.wikipedia.org/wiki/Regular_icosahedron" target="_top">
                https://en.wikipedia.org/wiki/Regular_icosahedron</a>

   @see Tetrahedron
   @see Cube
   @see Octahedron
   @see Dodecahedron
*/
public class Icosahedron extends Model
{
   /**
      Create a regular icosahedron with its center at
      the origin, having edge length
      <pre>{@code
        4/(1+sqrt(5)) = 1.2361,
      }</pre>
      and with its vertices on a sphere of radius
      <pre>{@code
        4/(1+sqrt(5)) * sin(2Pi/5) = 1.1756.
      }</pre>
   */
   public Icosahedron()
   {
      super();

      // Create the icosahedron's geometry.
      // It has 12 vertices, 30 edges, and 20 facess.
      double t = (1 + Math.sqrt(5))/2;  // golden ratio
      double r = 1/t;
      Vertex[] v = new Vertex[20];
      //https://en.wikipedia.org/wiki/Regular_icosahedron#Cartesian_coordinates
      // All cyclic permutations of (0, r, 1).
      v[0] = new Vertex(-r,  1,  0);
      v[1] = new Vertex( r,  1,  0);
      v[2] = new Vertex(-r, -1,  0);
      v[3] = new Vertex( r, -1,  0);
      v[4] = new Vertex( 0, -r,  1);
      v[5] = new Vertex( 0,  r,  1);
      v[6] = new Vertex( 0, -r, -1);
      v[7] = new Vertex( 0,  r, -1);
      v[8] = new Vertex( 1,  0, -r);
      v[9] = new Vertex( 1,  0,  r);
     v[10] = new Vertex(-1,  0, -r);
     v[11] = new Vertex(-1,  0,  r);
/*
      // These vertices create a icosahedron with edge length 2,
      // and vertices on a sphere of radius
      //    sqrt(10+2sqrt(5))/2 = 2sin(2Pi/5) = 1.90211.
      //https://en.wikipedia.org/wiki/Regular_icosahedron#Cartesian_coordinates
      // All cyclic permutations of (0, 1, t).
      v[0] = new Vertex(-1,  t,  0);
      v[1] = new Vertex( 1,  t,  0);
      v[2] = new Vertex(-1, -t,  0);
      v[3] = new Vertex( 1, -t,  0);
      v[4] = new Vertex( 0, -1,  t);
      v[5] = new Vertex( 0,  1,  t);
      v[6] = new Vertex( 0, -1, -t);
      v[7] = new Vertex( 0,  1, -t);
      v[8] = new Vertex( t,  0, -1);
      v[9] = new Vertex( t,  0,  1);
     v[10] = new Vertex(-t,  0, -1);
     v[11] = new Vertex(-t,  0,  1);
*/
      // Create 30 line segments.
      // To figure out the edges, look at the orthogonal projection in the z-direction.
      // https://en.wikipedia.org/wiki/Regular_icosahedron#Orthogonal_projections

      // The edge from v[0] to v[1] is the top horizontal edge.
      // The edge from v[2] to v[3] is the bottom horizontal edge.
      // The edge from v[4] to v[5] is the front vertical edge.
      // The edge from v[6] to v[7] is the back vertical edge.
      // The edge from v[8] to v[9] is the right horizontal edge.
      // The edge from v[10] to v[11] is the left horizontal edge.

      // Working, more or less, from the top down, front to back.
      addTriangle( v[ 0], v[ 1], v[ 5] );
      addTriangle( v[ 1], v[ 0], v[ 7] );
      addTriangle( v[ 0], v[ 5], v[11] );
      addTriangle( v[ 7], v[ 0], v[10] );
      addTriangle( v[ 0], v[10], v[11] );
      addTriangle( v[ 1], v[ 9], v[ 5] );
      addTriangle( v[ 8], v[ 1], v[ 7] );
      addTriangle( v[ 1], v[ 8], v[ 9] );
      addTriangle( v[ 5], v[ 4], v[11] );
      addTriangle( v[ 6], v[ 7], v[10] );
      addTriangle( v[ 5], v[ 9], v[ 4] );
      addTriangle( v[ 8], v[ 7], v[ 6] );
      addTriangle( v[10], v[11], v[ 2] );
      addTriangle( v[ 9], v[ 8], v[ 3] );
      addTriangle( v[11], v[ 4], v[ 2] );
      addTriangle( v[ 6], v[10], v[ 2] );
      addTriangle( v[ 4], v[ 9], v[ 3] );
      addTriangle( v[ 8], v[ 6], v[ 3] );
      addTriangle( v[ 2], v[ 4], v[ 3] );
      addTriangle( v[ 2], v[ 6], v[ 6] );
   }
}//Icosahedron
