/*

*/

package renderer.pipeline;
import  renderer.scene.*;
import  renderer.framebuffer.*;

import java.util.List;

/**
   Transform each {@link Vertex} of a {@link Model} from
   projected camera coordinates (i.e., image plane
   coordinates) to viewport coordinates.
<p>
   The image plane contains a view rectangle with
   <pre>{@code
        -1 <= x <= 1,
        -1 <= y <= 1,
   }</pre>
   The view rectangle has the important job of determining
   what part of the scene is visible to our renderer.
<p>
   In the previous pipeline stage, every vertex from the model was projected onto the image plane. We want only that part of each projected line segment contained in the view rectangle to be visible to our renderer and rasterized into the framebuffer's viewport. To prepare for the rasterization step, we need to convert each vertex's x and y coordinates


<pre>{@code
                                                                        (w+1/2,h+1/2)
        y-axis                       +----------------------------------------+
          |                          |                                        |
          |  (+1,+1)                 |                                        |
    +-----|-----+                    |                                        |
    |     |     |                    |                                        |
    |     |     |                    |                                        |
----------+----------- x-axis        |                                        |
    |     |     |                    |                                        |
    |     |     |                    |                                        |
    +-----|-----+                    |                                        |
 (-1,-1)  |                          |                                        |
          |                          |                                        |
                                     |                                        |
    View Rectangle                   +----------------------------------------+
  (in the view plane)            (1/2,1/2)
                                                    Logical Viewport
}</pre>



Notice how the view rectangle is a square but the logical viewport is a reactangle (though it may be a square).

The view rectangle always has -1.0 <= x <= 1.0 and -1.0 <= y <= 1.0.
The logical viewport always has 0.5 <= x <= w+0.5 and 0.5 <= y <= h+0.5 where w and h are the width and height of the FrameBuffer's current viewport.

If the aspect ratio, w/h, of the logical (and FrameBuffer) viewport is not 1, then the image in the view rectangle will be distorted when it is transformed to the logical viewport. (NOTE: Starting with Renderer 7,  the aspect ratio of the viewport will usually match the aspect ratio of the renderer's view volume, so the distortion caused by the viewport transformation will undo the distortion caused by the normalization transformation. If the view volume and viewport aspect ratios do not match, then the final image in the framebuffer's viewport will be a distortion of the image in the view volume's view rectangle.).



   For each {@link Vertex} object in each {@link Model} object,
   transform the {@link Vertex} object from image plane coordinates
   to viewport coordinates so that the view rectangle in the image
   plane with
   <pre>{@code
        -1 <= x <= 1,
        -1 <= y <= 1,
   }</pre>
   transforms into a viewport where
   <pre>{@code
       0.5 <= x < w + 0.5,
       0.5 <= y < h + 0.5,
   }</pre>
   where
   <pre>{@code
         w = number of horizontal pixels in the viewport,
         h = number of vertical pixels in the viewport.
   }</pre>
<p>
   The goal of this transformation is to put a logical pixel with
   integer coordinates at the center of each square physical pixel.
   The logical pixel with integer coordinates (m, n) represents the
   square pixel with
   <pre>{@code
     m - 0.5 <= x < m + 0.5,
     n - 0.5 <= y < n + 0.5.
   }</pre>
   Notice that logical pixel integer coordinates (m.n) have
   <pre>{@code
     1 <= m <= w
     1 <= n <= h.
   }</pre>
<p>
   Let us derive the formulas for the viewport transformation (we will
   derive the x-coordinate formula; the y-coordinate formula is similar).
<p>
   Let x_p denote an x-coordinate in the image plane and let x_vp denote
   an x-coordinate in the viewport. If a vertex is on the left edge of
   the view rectangle (with x_p = -1), then it should be transformed to
   the left edge of the viewport (with x_vp = 0.5). And if a vertex is on
   the right edge of the view rectangle (with x_p = 1), then it should be
   transformed to the right edge of the viewport (with x_vp =  w + 0.5).
   These two facts are all we need to know to find the linear function
   for the transformation of the x-coordinate.
<p>
   We need to calculate the slope m and intercept b of a linear function
   <pre>{@code
             x_vp = m * x_p + b
   }</pre>
   that converts image plane coordinates into viewport coordinates. We know,
   from what we said above about the left and right edges of the view
   rectangle, that
   <pre>{@code
              0.5 = (m * -1) + b,
          w + 0.5 = (m *  1) + b.
   }</pre>
   If we add these last two equations together we get
   <pre>{@code
            w + 1 = 2*b
   }</pre>
   or
   <pre>{@code
            b = (w + 1)/2.
   }</pre>
   If we use b to solve for m we have
   <pre>{@code
              0.5 = (m * -1) + (w + 1)/2
                1 = -2*m + w + 1
              2*m = w
                m = w/2.
   }</pre>
   So the linear transformation of the x-coordinate is
   <pre>{@code
          x_vp = (w/2) * x_p + (w+1)/2
               = 0.5 + w/2 * (x_p + 1).
   }</pre>
   The equivalent formula for the y-coordinate is
   <pre>{@code
          y_vp = 0.5 + h/2 * (y_p + 1).
   }</pre>
*/
public class Viewport
{
   /**
      For each {@link Primitive}, transform each {@link Vertex}
      from image plane coordinates to viewport coordinates.

      @param ls    {@link Primitive} to transform into viewport coordinates
      @param done  boolean array that keeps track of which vertices have been transformed
      @param fb    {@link FrameBuffer} that holds the current viewport
   */
   public static void viewport(Primitive p, boolean[] done, FrameBuffer fb)
   {
      // Get the viewport dimensions.
      int w = fb.getWidthVP();
      int h = fb.getHeightVP();

      // Transform the primitive's vertices from image
      // plane coordinates to viewport coordinates.
      for (int index : p.indexList)
      {
         // Transform only those vertices that
         // have not already been transformed.
         if (! done[index])
         {
            done[index] = true;
            Vertex v = p.model.vertexList.get( index );
            v.x = 0.5 + w/2.001 * (v.x + 1); //x_vp = 0.5 + w/2 * (x_p+1)
            v.y = 0.5 + h/2.001 * (v.y + 1); //y_vp = 0.5 + h/2 * (y_p+1)
            // NOTE: Notice the 2.001 fudge factor in the last two equations.
            // This is explained on page 142 of
            //    "Jim Blinn's Corner: A Trip Down The Graphics Pipeline"
            //     by Jim Blinn, 1996, Morgan Kaufmann Publishers.
         }
      }
   }
}
