/*

*/

package renderer.scene;

import java.util.List;
import java.util.ArrayList;
import java.awt.Color;

/**
   A {@code Primitive} is something that we can build
   geometric shapes out of (a "graphics primitive").
<p>
   We have three gemetric primitives, {@link LineSegment},
   {@link Face}, and {@link Triangle}.
<p>
   Notice that {@link LineSegment} is a 1-dimensional
   primitive while {@link Face} and {@link Triangle} are
   2-dimensional primitives.
<p>
   Each {@code Primitive} has a reference to a {@link Model}
   object that holds a {@link List} of {@link Vertex} objects
   that determing the geometry of the model.
<p>
   Each {@code Primitive} has a list of indices into its
   {@link Model}'s {@link List} of {@link Vertex} objects.
   These are the vertices that determine the primitive's
   geometry.
*/
/*
   NOTE: The Primitive class could be an inner class of
         the Model class. Then each Primitive object would
         automatically have acces to the actual Vertex list
         that the Primitive is indexing into. Here, we
         have every Primitve object hold a reference to
         the Model object that it is "part of".
*/
public abstract class Primitive
{
   // A Primitive object is made up of indices to vertices in a Model.
   public List<Integer> indexList;  // the indices for this primitive
   public Model model;              // this primitive's model object

   /**
      Construct an empty {@code Primitive}.

      @param model  {@link Model} containing the {@link Vertex} list for the new {@code Primitive}
   */
   protected Primitive(Model model)
   {
      this.model = model;
      this.indexList = new ArrayList<>();
   }


   /**
      Construct a {@code Primitive} from a {@link List} of integer indices.

      @param model  {@link Model} containing the {@link Vertex} list for the new {@code Primitive}
      @param vList  {@code List} of indices to put into the new {@code Primitive}
   */
   protected Primitive(Model model, List<Integer> iList)
   {
      this.model = model;
      this.indexList = iList;
   }


   /**
      Construct a {@code Primitive} from an array of integer indices.

      @param model   {@link Model} containing the {@link Vertex} list for the new {@code Primitive}
      @param vArray  array of indices to put into the new {@code Primitive}
   */
   protected Primitive(Model model, Integer... iArray)
   {
      this(model);

      for (Integer i : iArray)
      {
         indexList.add( i );
      }
   }


   /**
      Make a deep copy of this {@code Primitive} for another {@link Model}.

      @param newModel  the {@link Model} that the new {@code Primitive} will be part of
      @return an object that is a deep copy of this {@code Primitive} object
   */
   public abstract Primitive makeCopy(Model newModel);


   /**
      Add an index at the end of this {@code Primitive}'s index {@link List}.

      @param i  index to add to this {@code Primitive}
   */
   public void addVertex(int i)
   {
      indexList.add( i );  // auto boxing
   }


   /**
      Give this {@code Primitive} a uniform color.
      <p>
      NOTE: This works best when this {@code Primitive}
      object does not share its {@link Vertex} objects
      with another {@code Primitive} object.

      @param c  {@link Color} for this {@code Primitive}
   */
   public void setColor(Color c)
   {
      for (Integer index : indexList)
      {
         model.vertexList.get(index).setColor(c);
      }
   }


   /**
      Give this {@code Primitive} a uniform,
      but randomly chosen, {@link Color}.
      <p>
      NOTE: This works best when this {@code Primitive}
      object does not share its {@link Vertex} objects
      with another {@code Primitive} object.
   */
   public void setColorRandom()
   {
      if (! indexList.isEmpty())
      {
         model.vertexList.get(indexList.get(0)).setColorRandom();
         setColor( model.vertexList.get(indexList.get(0)).getColor() );
      }
   }


   /**
      Give each {@link Vertex} of this {@code Primitive}
      a randomly chosen {@link Color}.
      <p>
      NOTE: This works best when this {@code Primitive}
      object does not share its {@link Vertex} objects
      with another {@code Primitive} object.
   */
   public void setColorRainbow()
   {
      for (Integer index : indexList)
      {
         model.vertexList.get(index).setColorRandom();
      }
   }


   /**
      For debugging.

      @return {@link String} representation of this {@code Primitive} object
   */
   @Override
   public abstract String toString();
}
